-------------------------------------------------------------------------------
-- Cloud Serpent by Lockslap, US-Bleeding Hollow
-- http://ba-guild.com
-------------------------------------------------------------------------------
local CloudSerpent = LibStub("AceAddon-3.0"):NewAddon("Cloud Serpent", "AceEvent-3.0", "AceConsole-3.0")
local L	= LibStub("AceLocale-3.0"):GetLocale("Cloud Serpent", false)
if not TomTom then return end

CloudSerpent.name = GetAddOnMetadata("CloudSerpent", "Title")
CloudSerpent.version = GetAddOnMetadata("CloudSerpent", "Version")

local curZone, subZone
local waypoints = {}
local db
local defaults = {
	char = {
		enabled = true,
	}
}

-- zones that will activate the waypoints
local zones = {
	[L["Mistveil Sea"]] = true,
	[L["Windward Isle"]] = true,
	[L["The Arboretum"]] = true,
}

local repString	 = L["You are |cff%s%d/%d %s (%d%%)|r with the |cff1784d1Order of the Cloud Serpent|r."]
local requiredString = L["You require |cff1784d1%d|r %s%s, at |cff1784d1%d|r rep each, to reach Exalted"]
local iconString = "\124T%s:12\124t"
local enabledString = ("|cff00ff00%s|r"):format(L["Enabled"])
local disabledString = ("|cffff0000%s|r"):format(L["Disabled"])

local repStandings  = {
	[1] = L["Hated"],
	[2] = L["Hostile"],
	[3] = L["Unfriendly"],
	[4] = L["Neutral"],
	[5] = L["Friendly"],
	[6] = L["Honored"],
	[7] = L["Revered"],
	[8] = L["Exalted"],
}

function CloudSerpent:HandleChatCommand(args)
	local command = self:GetArgs(args)
	
	-- toggle the addon
	if not command or command == "" or command == "toggle" then
		db.enabled = not db.enabled
		self:Print((L["Waypoints have been %s"]):format(db.enabled == true and enabledString or disabledString))
		if not db.enabled and #waypoints > 0 then
			self:ClearWaypoints()
		elseif db.enabled == true then
			curZone = GetZoneText()
			subZone = GetSubZoneText()
			self:ClearWaypoints()
			self:UpdateWaypoints(curZone, subZone)
		end
		
	-- reload waypoints
	elseif command == "reload" then
		self:ClearWaypoints()
		curZone = GetZoneText()
		subZone = GetSubZoneText()
		if db.enabled then
			self:UpdateWaypoints(curZone, subZone)
		end
		
	-- rep status	
	elseif command == "status" then
		local _, link, _, _, _, _, _, _, _, texture, _ = GetItemInfo(89155)	-- onyx egg
		local icon = iconString:format(texture)
		local realValue, realMax, percent, numEggs = 0, 0, 0, 0
		local rep, race = 500, UnitRace("player")
		local guildLevel = IsInGuild() and GetGuildLevel() or nil
		local _, _, standing, barMin, barMax, barValue, _, _, _, _, _, _, _ = GetFactionInfoByID(1271)	-- cloud serpent's id is 1271
		local hexColor = ("%02x%02x%02x"):format(FACTION_BAR_COLORS[standing].r * 255, FACTION_BAR_COLORS[standing].g * 255, FACTION_BAR_COLORS[standing].b * 255)
		
		-- first determine if player is human
		if race:lower() == "human" then
			-- player has diplomacy
			rep = rep + (rep * 0.10)
		end
		
		-- now determine if player's guild has mr. popularity ranks 1 or 2
		if guildLevel and guildLevel >= 4 and guildLevel < 12 then
			-- rank 1
			rep = rep + (rep * 0.05)
		elseif guildLevel and guildLevel >= 12 then
			-- rank 2
			rep = rep + (rep * 0.10)
		end
		
		-- determine the rep values displayed on he reputation page, and the percentage completed
		realValue = barValue - barMin
		realMax = barMax - barMin
		percent = math.floor(realValue / realMax * 100)
		
		-- now determine how many more eggs are required to hit exalted
		numEggs = math.ceil((barMax - barValue) / rep)
		
		-- now print everything
		self:Print(repString:format(hexColor, realValue, realMax, repStandings[standing], percent))
		if standing ~= 8 then
			self:Print(requiredString:format(numEggs, icon, link, rep))
		end
		
	-- print help
	elseif command == "help" or command == "?" then
		self:Print((L["Cloud Serpent |cff00ff96v%.2f|r by |cff9382c9Lockslap|r"]):format(self.version))
		self:Print(L["|cff00ff96/cloudserpent|r or |cff00ff96/cs|r, Available Options: |cff00ff96help|r, |cff00ff96status|r, |cff00ff96toggle|r, or |cff00ff96reload|r."])
	
	end
end

function CloudSerpent:ClearWaypoints()
	if #waypoints == 0 then return end
	for _, point in pairs(waypoints) do
		TomTom:RemoveWaypoint(point)
	end
	waypoints = {}
	TomTom:ReloadWaypoints()
end

function CloudSerpent:UpdateWaypoints(zone, sub)
	if not db.enabled then
		if #waypoints > 0 then
			self:ClearWaypoints()
		end
		return
	end
	-- only need waypoints in the mistveil sea, windward isle, and the arboretum
	if zone == L["The Jade Forest"] and zones[sub] == true and #waypoints == 0 then
		for _, coord in pairs(self.coordinates) do
			-- add the waypoint to tomtom
			local opts = {
				title = ("%s\n(%.2f, %.2f)"):format(L["Onyx Egg Spawn Location"], coord.x, coord.y),
				persistent = false,
				minimap = true,
				world = true,
				crazy = true,
			}
			waypoints[#waypoints + 1] = TomTom:AddMFWaypoint(806, 0, coord.x / 100, coord.y / 100, opts)
		end
		--self:Print((L["Loaded |cff1784d1%d|r waypoints."]):format(#self.coordinates))
	elseif zone ~= L["The Jade Forest"] or not zones[sub] then
		self:ClearWaypoints()
	end
end

function CloudSerpent:ZoneChanged(event, ...)
	-- these will need to update regardless of which event fired
	curZone = GetZoneText()
	subZone = GetSubZoneText()
	if db.enabled == true then
		self:UpdateWaypoints(curZone, subZone)
	end
end

function CloudSerpent:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("CloudSerpentDB", defaults)
	db = self.db.char
	
	-- setup tomtom's arrow
	TomTom.profile.arrow.arrival = 5
	TomTom.profile.arrow.enable = true
	TomTom.profile.arrow.setclosest = true
end

function CloudSerpent:OnEnable()
	-- register the chat commands to toggle the addon
	self:RegisterChatCommand("cloudserpent", "HandleChatCommand")
	self:RegisterChatCommand("cs", "HandleChatCommand")
	
	curZone = GetZoneText()
	subZone = GetSubZoneText()
	if db.enabled == true then
		self:UpdateWaypoints(curZone, subZone)
	end
end

function CloudSerpent:OnDisable()
	self:UnregisterChatCommand("cloudserpent")
	self:UnregisterChatCommand("cs")
end

-- tracked events
CloudSerpent:RegisterEvent("ZONE_CHANGED", "ZoneChanged")
CloudSerpent:RegisterEvent("ZONE_CHANGED_INDOORS", "ZoneChanged")
CloudSerpent:RegisterEvent("ZONE_CHANGED_NEW_AREA", "ZoneChanged")

-- the coordinates of known egg locations
CloudSerpent.coordinates = {
	{x = 65.08, y = 26.20},
	{x = 66.64, y = 28.69},
	{x = 67.12, y = 24.94},
	{x = 65.18, y = 24.77},
	{x = 67.05, y = 33.81},
	{x = 66.61, y = 31.58},
	{x = 66.54, y = 32.20},
	{x = 63.09, y = 29.31},
	{x = 69.65, y = 31.64},
	{x = 70.53, y = 28.99},
	{x = 61.89, y = 30.20},
	{x = 66.18, y = 33.43},
	{x = 62.40, y = 32.36},
	{x = 65.63, y = 25.07},
	{x = 64.18, y = 23.92},
	{x = 63.39, y = 21.62},
	{x = 62.98, y = 21.95},
	{x = 62.37, y = 32.84},
	{x = 62.17, y = 31.75},
	{x = 62.19, y = 31.21},
	{x = 67.02, y = 32.68},
	{x = 62.46, y = 28.77},
	{x = 62.21, y = 29.29},
	{x = 62.00, y = 29.54},
	{x = 63.73, y = 21.90},
	{x = 62.30, y = 28.12},
	{x = 62.71, y = 29.78},
	{x = 66.43, y = 29.55},
	{x = 65.18, y = 24.77},
	{x = 65.24, y = 23.91},
	{x = 70.33, y = 28.63},
	{x = 67.06, y = 30.25},
	{x = 66.00, y = 30.71},
}